import QtQuick 2.0
import QtQml 2.12
import QtQuick.Layouts 1.12
import QtQuick.Controls 2.5
import QtQml.Models 2.2
import QtQuick.Window 2.3
import MyGeneralSetting 1.0

Rectangle {
    id: dialogItem

    property alias dialogItemX: dialogItem.x
    property alias dialogItemY: dialogItem.y

    property alias dialogVisible: dialog.visible
    property alias dialogWidth: contentRect.width
    property alias dialogHeight: contentRect.height

    property alias imageWidth: image.width
    property alias imageHeight: image.height
    property alias imageSource: image.source

    property string imagePath
    property string execName: ""
    property double dialogOpacity : 0.2
    property bool dragSwitch: false
    property string dragName

    Popup {
        id: dialog
        padding: 0
        width: 0
        height: 0
        visible: true
        closePolicy: Popup.NoAutoClose
        anchors.centerIn: parent.Center

        Rectangle {
            id: contentRect
            width: Style.widthScaleFactor * 55
            height: Style.heightScaleFactor * 55
            radius: 100
            color: Qt.rgba(26/256, 26/256, 26/256, dialogOpacity)
            anchors.centerIn: parent.Center

            MouseArea {
                id: dragArea
                anchors.fill: parent
                Drag.active: dragArea.drag.active
                drag.target: dragSwitch == false? undefined : dialogItem

                onClicked: {
                    image.clickEvent(execName)
                    //点击悬浮按钮也可退出编辑状态
//                    itemView.titleFocusChanged()
                }
                onPressed: {
                    contentRect.color = Qt.rgba(26/256, 26/256, 26/256, dialogOpacity == 0.2? 0.4:0.2)
                }
                onReleased: {
                    contentRect.color = Qt.rgba(26/256, 26/256, 26/256, dialogOpacity)
                }
            }

            Image {
                id: image
                width: Style.widthScaleFactor * 40
                height: Style.heightScaleFactor * 40
                sourceSize.width: imageWidth; sourceSize.height: imageHeight
                anchors.centerIn: parent
                smooth: true
                source: imagePath
                //点击设置按钮发出信号
                signal clickEvent(string exec)

                CGeneralSetting {
                    id: cGeneralSetting
                }
            }
        }
    }

    Component.onCompleted: {
        // 1. Qml对象的信号关联到Cpp的槽函数
        image.clickEvent.connect(cGeneralSetting.openGeneralSetting)
    }
}


